@@ -1,4 +1,13 @@
 #!/usr/bin/python -u
+"""
+updb.py - the core of rpcpdb.
+
+Contains mixin methods for RPC servers to support client-initiated
+function debugging & tracing
+
+Copyright 2010-2013 Ben Bass <ben.bass@codedstructure.net>
+Licence: MIT - http://www.opensource.org/licenses/mit-license.php
+"""
 
 try:
     from inspect import getcallargs, getargspec
@@ -14,6 +23,8 @@ import tempfile
 import threading
 import code
 
+from rpcpdb.func_trace import Tracer
+
 
 class UPdb(pdb.Pdb):
     def __init__(self, sock_path=None, level=0, force=False):
@@ -117,9 +128,10 @@ class UPdb_mixin(object):
     will trigger a pdb breakpoint (currently with
     interaction over a unix socket)
     """
-    # this is a map from function name to
-    # (function, unix_socket_path) pairs.
+    # these are maps from function name to
+    # (function, unix_socket_path/trace_path) pairs.
     _updb_debug_func = {}
+    _updb_trace_func = {}
 
     def debug_func(self, f, once=True, force=True,
                    match_criteria=None, ignore_count=0):
@@ -187,6 +199,74 @@ class UPdb_mixin(object):
         if f in self._updb_debug_func:
             setattr(self, f, self._updb_debug_func.pop(f)[0])
 
+    def trace_func(self, f, tracepath, once=True,
+                   match_criteria=None, ignore_count=0):
+        """
+        @param f: function name
+        @param tracepath: pathname to write trace to
+        @param once: one-shot debug - untrace after first hit
+        @param match_criteria: dictionary of param name/value matches
+        @param ignore_count: don't trigger for ignore_count matches
+        @return: path to trace file
+        """
+        if f in self._updb_trace_func:
+            return self._updb_trace_func[f][1]
+        else:
+            func = getattr(self, f)
+            self._updb_trace_func[f] = (func, tracepath)
+
+            def arg_match(o, k):
+                # check whether function f is called with
+                # parameters matching the given match_criteria
+                if not match_criteria:
+                    # we always match if we have no match criteria
+                    return True
+                called_with = getcallargs(func, *o, **k)
+                # insert anything passed using **kwargs etc as if it were
+                # a standard named parameter (for the purposes of comparison)
+                fn_keyword_arg_name = getargspec(func)[2]
+                if fn_keyword_arg_name is not None:
+                    for kw_k, kw_v in called_with.get(fn_keyword_arg_name,
+                                                      {}).items():
+                        called_with[kw_k] = kw_v
+
+                match = True
+                # All things in match_criteria must have exact matches
+                # in called_with for us to match things.
+                for key, val in match_criteria.items():
+                    # note short-circuit logic here
+                    if key not in called_with or called_with[key] != val:
+                        match = False
+
+                return match
+
+            def debug_check(*o, **k):
+                if arg_match(o, k):
+                    if debug_check._ignore_count <= 0:
+                        if once:
+                            self.untrace_func(f)
+                        old_tracer = sys.gettrace()
+                        sys.settrace(Tracer(tracepath))
+                        try:
+                            return func(*o, **k)
+                        finally:
+                            sys.settrace(old_tracer)
+                    debug_check._ignore_count -= 1
+                # we're not tracing you, this time...
+                return func(*o, **k)
+            # use a function attribute to store the skip count;
+            # decrement this on each matched call. Trigger at <= 0
+            debug_check._ignore_count = ignore_count
+
+            setattr(self, f, debug_check)
+
+    def untrace_func(self, f):
+        """
+        don't trace this function anymore.
+        """
+        if f in self._updb_trace_func:
+            setattr(self, f, self._updb_trace_func.pop(f)[0])
+
     def debug_interactive_console(self):
         """
         start an interactive console in a new thread
